@@ -17,7 +17,7 @@ along with GNU Emacs; see the file COPYING.  If not, write to the Free
 Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 02111-1307, USA.  */
 
-/* Contributed by Andrew Choi (akochoi@users.sourceforge.net).  */
+/* Contributed by Andrew Choi (akochoi@mac.com).  */
 
 #ifndef _ALLOCA_H_
 #define _ALLOCA_H_
@@ -27,3 +27,6 @@ void *__alloca(size_t size);
 #endif
 
 #endif
+
+/* arch-tag: 8d4114d1-633e-49e7-8f18-b5648f9c4511
+   (do not change this comment) */
